@@ -29,25 +29,29 @@
 fields"""
 
 def notEditableAdmin(original_admin):
-  """Turn all fields visualized with original_admin into read only fields
-:param original_admin: an implementation of ObjectAdmin
+    """Turn all fields visualized with original_admin into read only fields
+  :param original_admin: an implementation of ObjectAdmin
 
-usage ::
+  usage ::
 
-  class Movie(Entity):
-    name = Field(Unicode(50))
-    
-    class Admin(EntityAdmin):
-      list_display = ['name']
-      
-    Admin = notEditableAdmin(Admin)
-  """
-     
-  class newAdmin(original_admin):
-    
-    def getFieldAttributes(self, field_name):
-      attribs = original_admin.getFieldAttributes(self, field_name)
-      attribs['editable'] = False
-      return attribs
-      
-  return newAdmin  
\ No newline at end of file
+    class Movie(Entity):
+      name = Field(Unicode(50))
+
+      class Admin(EntityAdmin):
+        list_display = ['name']
+
+      Admin = notEditableAdmin(Admin)
+    """
+
+    class NewAdmin(original_admin):
+
+#    def get_related_entity_admin(self, entity):
+#      admin = original_admin.get_related_entity_admin(self, entity)
+#      return notEditableAdmin(admin)
+
+        def get_field_attributes(self, field_name):
+            attribs = original_admin.get_field_attributes(self, field_name)
+            attribs['editable'] = False
+            return attribs
+
+    return NewAdmin
