@@ -28,17 +28,17 @@
 """Functions and widget to represent exceptions to the user"""
 
 def model_thread_exception_message_box(exception_info):
-  """Display an exception that occurred in the model thread in a message box,
-use this function as the exception argument in the model thread's post function
-to represent the exception to the user
-  
-:param exception_info: a tuple containing the exception that was thrown and the
-model thread in which the exception was thrown  
-"""
-  from PyQt4 import QtGui
-  exc, model_thread = exception_info
-  msgBox = QtGui.QMessageBox(QtGui.QMessageBox.Warning,
-                             'Exception', 'An unexpected event occurred')
-  msgBox.setInformativeText(str(exc))
-  msgBox.setDetailedText(model_thread.traceback())
-  msgBox.exec_()
+    """Display an exception that occurred in the model thread in a message box,
+  use this function as the exception argument in the model thread's post function
+  to represent the exception to the user
+    
+  :param exception_info: a tuple containing the exception that was thrown and the
+  model thread in which the exception was thrown  
+  """
+    from PyQt4 import QtGui
+    exc, traceback = exception_info
+    msgBox = QtGui.QMessageBox(QtGui.QMessageBox.Warning,
+                               'Exception', 'An unexpected event occurred')
+    msgBox.setInformativeText(unicode(exc))
+    msgBox.setDetailedText(traceback)
+    msgBox.exec_()
