@@ -1,13 +1,11 @@
 # -*- coding: utf-8 -*-
-from PyQt4 import QtGui, QtCore
+from PyQt4 import QtGui
 
 import logging
-import settings
 import unittest
-import sys
 import os
 
-from camelot.test import *
+from camelot.test import ModelThreadTestCase, EntityViewsTest, SchemaTest
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 
@@ -17,102 +15,103 @@ static_images_path = os.path.join(os.path.dirname(__file__), '..', 'doc', 'sphin
 
 def create_getter(getable):
   
-  def getter():
-    return getable
-  return getter
+    def getter():
+        return getable
+    
+    return getter
         
-class ProxyEntityTest(ModelThreadTestCase):
-  """Test the functionality of the proxies to perform CRUD operations on stand
-  alone data"""
-  def setUp(self):
-    ModelThreadTestCase.setUp(self)
-    from camelot.model.authentication import Person, PartyAddressRoleType
-    from camelot.view.proxy.queryproxy import QueryTableProxy
-    from camelot.view.application_admin import ApplicationAdmin
-    self.app_admin = ApplicationAdmin()
-    self.block = self.mt.post_and_block
-    person_admin = self.app_admin.getEntityAdmin(Person)
-    party_address_role_type_admin = self.app_admin.getEntityAdmin(PartyAddressRoleType)
-    self.person_proxy = QueryTableProxy(person_admin, lambda:Person.query, person_admin.getFields)
-    self.party_address_role_type_proxy = QueryTableProxy(party_address_role_type_admin, lambda:PartyAddressRoleType.query, party_address_role_type_admin.getFields)
-    # get the columns of the proxy
-    self.person_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.person_proxy.getColumns())))
-    self.rows_before_insert = 0
-    self.rows_after_insert = 0
-    self.rows_after_delete = 0
-    self.new_person = None
-    # make sure nothing is running in the model thread any more
-    self.block(lambda:None)
-  def listPersons(self):
-    from camelot.model.authentication import Person
-    
-    def create_list():
-      print '<persons>'
-      for p in Person.query.all():
-        print p.id, unicode(p)
-      print '</persons>'
-    
-    self.block(create_list)
-  def numberOfPersons(self):
-    return self.block(lambda:self.person_proxy.getRowCount())
-  def insertNewPerson(self, valid):
-    from camelot.model.authentication import Person
-    self.rows_before_insert = self.numberOfPersons()
-    self.new_person = self.block(lambda:Person(first_name={True:u'test', False:None}[valid], last_name='test'))
-    self.person_proxy.insertRow(0, create_getter(self.new_person))
-    self.rows_after_insert = self.numberOfPersons()
-    self.assertTrue( self.rows_after_insert > self.rows_before_insert)
-  def updateNewPerson(self):
-    rows_before_update = self.numberOfPersons()
-    self.assertNotEqual( self.block(lambda:self.new_person.last_name), u'Testers')
-    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-    self.person_proxy.setData(index, lambda:u'Testers')
-    self.assertEqual( self.block(lambda:self.new_person.last_name), u'Testers')
-    self.assertEqual(rows_before_update, self.numberOfPersons())
-  def updateNewPersonToValid(self):
-    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
-    self.person_proxy.setData(index, lambda:u'test')
-    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-    self.person_proxy.setData(index, lambda:u'test')      
-  def updateNewPersonToInvalid(self):
-    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
-    self.person_proxy.setData(index, lambda:None)
-    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
-    self.person_proxy.setData(index, lambda:None)      
-  def deleteNewPerson(self):
-    self.person_proxy.removeRow(self.rows_before_insert)
-    self.rows_after_delete = self.numberOfPersons()
-    self.assertTrue( self.rows_after_delete < self.rows_after_insert )
-  def testCreateDataWithoutRequiredFields(self):
-    """Create a record that has no required fields, so the creation
-    should be instantaneous as opposed to delayed until all required fields are filled"""
-    from camelot.model.authentication import PartyAddressRoleType
-    rows_before_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
-    new_party_address_role_type_proxy = self.block(lambda:PartyAddressRoleType())
-    self.party_address_role_type_proxy.insertRow(0, create_getter(new_party_address_role_type_proxy))
-    self.assertNotEqual( self.block(lambda:new_party_address_role_type_proxy.id), None )
-    rows_after_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
-    self.assertTrue( rows_after_insert > rows_before_insert)      
-  def testCreateUpdateDeleteValidData(self):
-    self.insertNewPerson(valid=True)
-    self.assertTrue( self.block(lambda:self.new_person.id) != None )
-    self.updateNewPerson()
-    self.deleteNewPerson()
-  def testCreateUpdateDeleteInvalidData(self):
-    self.insertNewPerson(valid=False)
-    self.assertTrue( self.block(lambda:self.new_person.id) == None )
-    self.updateNewPerson()
-    self.deleteNewPerson()
-  def testCreateInvalidDataUpdateToValidAndDelete(self):
-    self.insertNewPerson(valid=False)
-    self.assertTrue( self.block(lambda:self.new_person.id) == None )
-    self.updateNewPersonToValid()
-    self.assertTrue( self.block(lambda:self.new_person.id) != None )
-    self.deleteNewPerson()
-  def testCreateValidDataUpdateToInvalidAndDelete(self):
-    self.insertNewPerson(valid=False)
-    self.updateNewPersonToInvalid()
-    self.deleteNewPerson()
+#class ProxyEntityTest(ModelThreadTestCase):
+#  """Test the functionality of the proxies to perform CRUD operations on stand
+#  alone data"""
+#  def setUp(self):
+#    ModelThreadTestCase.setUp(self)
+#    from camelot.model.authentication import Person, PartyAddressRoleType
+#    from camelot.view.proxy.queryproxy import QueryTableProxy
+#    from camelot.view.application_admin import ApplicationAdmin
+#    self.app_admin = ApplicationAdmin()
+#    self.block = self.mt.post_and_block
+#    person_admin = self.app_admin.getEntityAdmin(Person)
+#    party_address_role_type_admin = self.app_admin.getEntityAdmin(PartyAddressRoleType)
+#    self.person_proxy = QueryTableProxy(person_admin, lambda:Person.query, person_admin.get_fields)
+#    self.party_address_role_type_proxy = QueryTableProxy(party_address_role_type_admin, lambda:PartyAddressRoleType.query, party_address_role_type_admin.get_fields)
+#    # get the columns of the proxy
+#    self.person_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.person_proxy.getColumns())))
+#    self.rows_before_insert = 0
+#    self.rows_after_insert = 0
+#    self.rows_after_delete = 0
+#    self.new_person = None
+#    # make sure nothing is running in the model thread any more
+#    self.block(lambda:None)
+#  def listPersons(self):
+#    from camelot.model.authentication import Person
+#    
+#    def create_list():
+#      print '<persons>'
+#      for p in Person.query.all():
+#        print p.id, unicode(p)
+#      print '</persons>'
+#    
+#    self.block(create_list)
+#  def numberOfPersons(self):
+#    return self.block(lambda:self.person_proxy.getRowCount())
+#  def insertNewPerson(self, valid):
+#    from camelot.model.authentication import Person
+#    self.rows_before_insert = self.numberOfPersons()
+#    self.new_person = self.block(lambda:Person(first_name={True:u'test', False:None}[valid], last_name='test'))
+#    self.person_proxy.insertRow(0, create_getter(self.new_person))
+#    self.rows_after_insert = self.numberOfPersons()
+#    self.assertTrue( self.rows_after_insert > self.rows_before_insert)
+#  def updateNewPerson(self):
+#    rows_before_update = self.numberOfPersons()
+#    self.assertNotEqual( self.block(lambda:self.new_person.last_name), u'Testers')
+#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
+#    self.person_proxy.setData(index, lambda:u'Testers')
+#    self.assertEqual( self.block(lambda:self.new_person.last_name), u'Testers')
+#    self.assertEqual(rows_before_update, self.numberOfPersons())
+#  def updateNewPersonToValid(self):
+#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
+#    self.person_proxy.setData(index, lambda:u'test')
+#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
+#    self.person_proxy.setData(index, lambda:u'test')      
+#  def updateNewPersonToInvalid(self):
+#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['first_name'])
+#    self.person_proxy.setData(index, lambda:None)
+#    index = self.person_proxy.index(self.rows_before_insert, self.person_columns['last_name'])
+#    self.person_proxy.setData(index, lambda:None)      
+#  def deleteNewPerson(self):
+#    self.person_proxy.removeRow(self.rows_before_insert)
+#    self.rows_after_delete = self.numberOfPersons()
+#    self.assertTrue( self.rows_after_delete < self.rows_after_insert )
+#  def testCreateDataWithoutRequiredFields(self):
+#    """Create a record that has no required fields, so the creation
+#    should be instantaneous as opposed to delayed until all required fields are filled"""
+#    from camelot.model.authentication import PartyAddressRoleType
+#    rows_before_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
+#    new_party_address_role_type_proxy = self.block(lambda:PartyAddressRoleType())
+#    self.party_address_role_type_proxy.insertRow(0, create_getter(new_party_address_role_type_proxy))
+#    self.assertNotEqual( self.block(lambda:new_party_address_role_type_proxy.id), None )
+#    rows_after_insert = self.block(lambda:self.party_address_role_type_proxy.getRowCount())
+#    self.assertTrue( rows_after_insert > rows_before_insert)      
+#  def testCreateUpdateDeleteValidData(self):
+#    self.insertNewPerson(valid=True)
+#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
+#    self.updateNewPerson()
+#    self.deleteNewPerson()
+#  def testCreateUpdateDeleteInvalidData(self):
+#    self.insertNewPerson(valid=False)
+#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
+#    self.updateNewPerson()
+#    self.deleteNewPerson()
+#  def testCreateInvalidDataUpdateToValidAndDelete(self):
+#    self.insertNewPerson(valid=False)
+#    self.assertTrue( self.block(lambda:self.new_person.id) == None )
+#    self.updateNewPersonToValid()
+#    self.assertTrue( self.block(lambda:self.new_person.id) != None )
+#    self.deleteNewPerson()
+#  def testCreateValidDataUpdateToInvalidAndDelete(self):
+#    self.insertNewPerson(valid=False)
+#    self.updateNewPersonToInvalid()
+#    self.deleteNewPerson()
 
                 
 #class ProxyOneToManyTest(ProxyEntityTest):
@@ -124,7 +123,7 @@ class ProxyEntityTest(ModelThreadTestCase):
 #    from camelot.view.proxy.queryproxy import QueryTableProxy
 #    from camelot.model.authentication import Country, City, Address, PartyAddress
 #    party_address_admin = self.app_admin.getEntityAdmin(PartyAddress)
-#    self.party_address_proxy = QueryTableProxy(party_address_admin, PartyAddress.query, party_address_admin.getFields)
+#    self.party_address_proxy = QueryTableProxy(party_address_admin, PartyAddress.query, party_address_admin.get_fields)
 #    self.party_address_columns = dict((c[0],i) for i,c in enumerate(self.block(lambda:self.party_address_proxy.getColumns())))
 #    self.country = self.block(lambda:Country(code=u'BE', name=u'Belgium'))
 #    self.city = self.block(lambda:City(code=u'2000', name=u'Antwerp', country=self.country))
@@ -536,10 +535,26 @@ class FormTest(ModelThreadTestCase):
       'director':(QLabel('Director'), TextLineEditor(parent=None, editable=True)),
       'release_date':(QLabel('Release date'), DateEditor(parent=None, editable=True)),
     }
+    from elixir import entities
+    self.entities = [e for e in entities]
+    from camelot.admin.application_admin import ApplicationAdmin
+    from camelot.model.authentication import Person
+    self.app_admin = ApplicationAdmin()
+    self.person_entity = Person
+    self.collection_getter = lambda:[Person()]   
+    
+  def tearDown(self):
+    #
+    # The global list of entities should remain clean for subsequent tests
+    #
+    from elixir import entities
+    for e in entities:
+      if e not in self.entities:
+        entities.remove(e)
     
   def test_form(self):
-    form = forms.Form(['title', 'short_description', 'director', 'release_date'])
-    self.grab_widget(form.render(self.widgets))
+    from snippet.form.simple_form import Movie
+    self.grab_widget(Movie.Admin.form_display.render(self.widgets))
     
   def test_tab_form(self):
     form = forms.TabForm([('First tab', ['title', 'short_description']),
@@ -560,7 +575,22 @@ class FormTest(ModelThreadTestCase):
     
   def test_hbox_form(self):
     form = forms.HBoxForm([['title', 'short_description'], ['director', 'release_date']])
-    self.grab_widget(form.render(self.widgets))    
+    self.grab_widget(form.render(self.widgets))
+    
+  def test_nested_form(self):
+      from snippet.form.nested_form import Admin
+      person_admin = Admin(self.app_admin, self.person_entity)
+      self.grab_widget( person_admin.create_new_view() )
+      
+  def test_inherited_form(self):
+      from snippet.form.inherited_form import InheritedAdmin
+      person_admin = InheritedAdmin(self.app_admin, self.person_entity)
+      self.grab_widget( person_admin.create_new_view() )
+      
+  def test_custom_layout(self):
+      from snippet.form.custom_layout import Admin
+      person_admin = Admin(self.app_admin, self.person_entity)
+      self.grab_widget( person_admin.create_new_view() )      
                           
 from camelot.admin import form_action
   
@@ -585,6 +615,11 @@ class DelegateTest(unittest.TestCase):
     
   def setUp(self):
     self.kwargs = dict(editable=True)
+    self.option = QtGui.QStyleOptionViewItem()
+    # set version to 5 to indicate the widget will appear on a
+    # a form view and not on a table view, so it should not
+    # set its background
+    self.option.version = 5
 
   def grab_delegate(self, delegate, data, suffix='editable'):
     import sys
@@ -638,57 +673,57 @@ class DelegateTest(unittest.TestCase):
     delegate = self.delegates.PlainTextDelegate(parent=None,
                                                 length=30,
                                                 editable=True)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertEqual(editor.maxLength(), 30)
     self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
     self.grab_delegate(delegate, 'Plain Text')
     delegate = self.delegates.PlainTextDelegate(parent=None,
                                                 length=20,
                                                 editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.TextLineEditor))
     self.grab_delegate(delegate, 'Plain Text', 'disabled')
     
   def testTextEditDelegate(self):
     from PyQt4.QtGui import QTextEdit
     delegate = self.delegates.TextEditDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, QTextEdit))
     self.grab_delegate(delegate, 'Plain Text')
     delegate = self.delegates.TextEditDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, QTextEdit))
     self.grab_delegate(delegate, 'Plain Text', 'disabled')
 
   def testRichTextDelegate(self):
     delegate = self.delegates.RichTextDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
     self.grab_delegate(delegate, '<b>Rich Text</b>')
     delegate = self.delegates.RichTextDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.RichTextEditor))
     self.grab_delegate(delegate, '<b>Rich Text</b>', 'disabled')
     
   def testBoolDelegate(self):
     delegate = self.delegates.BoolDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.BoolEditor))
     self.grab_delegate(delegate, True)
     delegate = self.delegates.BoolDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.BoolEditor))
     self.grab_delegate(delegate, True, 'disabled')
   
   def testDateDelegate(self):
     from datetime import date
     delegate = self.delegates.DateDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.DateEditor))
     date = date.today()
     self.grab_delegate(delegate, date)
     delegate = self.delegates.DateDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.DateEditor))
     date = date.today()
     self.grab_delegate(delegate, date, 'disabled')
@@ -697,35 +732,35 @@ class DelegateTest(unittest.TestCase):
   def testDateTimeDelegate(self):
     from datetime import datetime
     delegate = self.delegates.DateTimeDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
     DateTime = datetime.now()
     self.grab_delegate(delegate, DateTime)
     delegate = self.delegates.DateTimeDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.DateTimeEditor))
     self.grab_delegate(delegate, DateTime, 'disabled')
 
   def testTimeDelegate(self):
     from datetime import time
     delegate = self.delegates.TimeDelegate(parent=None, editable=True)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.TimeEditor))
     time = time(10, 30, 15)
     self.grab_delegate(delegate, time)
     delegate = self.delegates.TimeDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.TimeEditor))
     #time = time(10, 30, 15)
     self.grab_delegate(delegate, time, 'disabled')
     
   def testIntegerDelegate(self):
     delegate = self.delegates.IntegerDelegate(parent=None, editable=True)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
     self.grab_delegate(delegate, 3)
     delegate = self.delegates.IntegerDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.IntegerEditor))
     self.grab_delegate(delegate, 0, 'disabled')
 
@@ -740,13 +775,13 @@ class DelegateTest(unittest.TestCase):
   def testFloatDelegate(self):
     from camelot.core.constants import camelot_minfloat, camelot_maxfloat
     delegate = self.delegates.FloatDelegate(parent=None, suffix='euro', editable=True)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.FloatEditor))
     self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
     self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
     self.grab_delegate(delegate, 3.145)
     delegate = self.delegates.FloatDelegate(parent=None, prefix='prefix', editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.FloatEditor))
     self.assertEqual(editor.spinBox.minimum(), camelot_minfloat)
     self.assertEqual(editor.spinBox.maximum(), camelot_maxfloat)
@@ -754,65 +789,65 @@ class DelegateTest(unittest.TestCase):
 
   def testColoredFloatDelegate(self):
     delegate = self.delegates.ColoredFloatDelegate(parent=None, precision=3, editable=True)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
     self.grab_delegate(delegate, 3.14456)
     delegate = self.delegates.ColoredFloatDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ColoredFloatEditor))
     self.grab_delegate(delegate, 3.1, 'disabled')
   
   def testStarDelegate(self):
     delegate = self.delegates.StarDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(delegate.maximum, 5)
     self.assertTrue(isinstance(editor, self.editors.StarEditor))
     self.grab_delegate(delegate, 5)
     delegate = self.delegates.StarDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(delegate.maximum, 5)
     self.assertTrue(isinstance(editor, self.editors.StarEditor))
     self.grab_delegate(delegate, 5, 'disabled')
     
   def testSmileyDelegate(self):
     delegate = self.delegates.SmileyDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
     self.grab_delegate(delegate, 'face-glasses')
     delegate = self.delegates.SmileyDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.SmileyEditor))
     self.grab_delegate(delegate, 'face-glasses', 'disabled')
 
   def testFileDelegate(self):
     delegate = self.delegates.FileDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.FileEditor))
 
   def testColorDelegate(self):
     delegate = self.delegates.ColorDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ColorEditor))
     color = [255, 255, 0]
     self.grab_delegate(delegate, color)
     delegate = self.delegates.ColorDelegate(parent=None, editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ColorEditor))
     self.grab_delegate(delegate, color, 'disabled')
   
   def testCodeDelegate(self):
     delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA'], **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.CodeEditor))
     self.grab_delegate(delegate, ['76','AB'])
     delegate = self.delegates.CodeDelegate(parent=None, parts=['99','AA', '99', 'AA'], editable=False)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.CodeEditor))
     self.grab_delegate(delegate, ['76','AB', '12', '34'], 'disabled')
     
   def testCurrencyDelegate(self):
     delegate = self.delegates.CurrencyDelegate(parent=None, suffix='euro', **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.FloatEditor))
     self.grab_delegate(delegate, 1000000.12)
     delegate = self.delegates.CurrencyDelegate(parent=None, prefix='prefix', editable=False)
@@ -823,7 +858,7 @@ class DelegateTest(unittest.TestCase):
     delegate = self.delegates.ComboBoxDelegate(parent=None,
                                                choices=CHOICES,
                                                **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ChoicesEditor))
     self.grab_delegate(delegate, 1)
     delegate = self.delegates.ComboBoxDelegate(parent=None,
@@ -833,13 +868,13 @@ class DelegateTest(unittest.TestCase):
 
   def testImageDelegate(self):
     delegate = self.delegates.ImageDelegate(parent=None, **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.ImageEditor))
 
   def testVirtualAddressDelegate(self):
     delegate = self.delegates.VirtualAddressDelegate(parent=None,
                                                      **self.kwargs)
-    editor = delegate.createEditor(None, None, None)
+    editor = delegate.createEditor(None, self.option, None)
     self.assertTrue(isinstance(editor, self.editors.VirtualAddressEditor))
     self.grab_delegate(delegate, ('email', 'project-camelot@conceptive.be'))
     delegate = self.delegates.VirtualAddressDelegate(parent=None, editable=False)
@@ -861,30 +896,36 @@ class FilterTest(ModelThreadTestCase):
     self.grab_widget(filter.render(None, 'Organization', [('Nokia',None), ('Apple',None)]))
     
 class ControlsTest(ModelThreadTestCase):
-  """Test some basic controls"""
-  
-  images_path = static_images_path
-  
-  def setUp(self):
-    super(ControlsTest, self).setUp()
-    from camelot.admin.application_admin import ApplicationAdmin
-    self.app_admin = ApplicationAdmin()
-        
-  def test_table_view(self):
-    from camelot.view.controls.tableview import TableView
-    from camelot.model.authentication import Person
-    widget = TableView(self.app_admin.get_entity_admin(Person))
-    self.grab_widget(widget)
-  
-  def test_navigation_pane(self):
-    from camelot.view.controls import navpane
-    widget = navpane.NavigationPane(self.app_admin)
-    self.grab_widget(widget)
+    """Test some basic controls"""
     
-  def test_main_window(self):
-    from camelot.view.mainwindow import MainWindow
-    widget = MainWindow(self.app_admin)
-    self.grab_widget(widget)
+    images_path = static_images_path
+    
+    def setUp(self):
+        super(ControlsTest, self).setUp()
+        from camelot.admin.application_admin import ApplicationAdmin
+        self.app_admin = ApplicationAdmin()
+          
+    def test_table_view(self):
+        from camelot.view.controls.tableview import TableView
+        from camelot.model.authentication import Person
+        widget = TableView(self.app_admin.get_entity_admin(Person))
+        self.grab_widget(widget)
+    
+    def test_navigation_pane(self):
+        from camelot.view.controls import navpane
+        widget = navpane.NavigationPane(self.app_admin)
+        self.grab_widget(widget)
+      
+    def test_main_window(self):
+        from camelot.view.mainwindow import MainWindow
+        widget = MainWindow(self.app_admin)
+        self.grab_widget(widget)
+        
+    def test_status_bar(self):
+        from camelot.view.controls.statusbar import StatusBar
+        status_bar = StatusBar(None)
+        status_bar.busy_widget.set_busy(True)
+        self.grab_widget(status_bar)
     
 class CamelotEntityViewsTest(EntityViewsTest):
   """Test the views of all the Entity subclasses"""
@@ -893,57 +934,87 @@ class CamelotEntityViewsTest(EntityViewsTest):
   
 class SnippetsTest(ModelThreadTestCase):
     
-  images_path = static_images_path
-  
-  def test_fields_with_actions(self):
-    from snippet.fields_with_actions import Coordinate
-    from camelot.view.proxy.collection_proxy import CollectionProxy
-    coordinate = Coordinate()
-    admin = Coordinate.Admin(None, Coordinate)
-    proxy = CollectionProxy(admin, lambda:[coordinate], admin.getFields )
-    form = admin.create_form_view('Coordinate', proxy, 0, None)
-    self.grab_widget(form)
+    images_path = static_images_path
     
-  def test_fields_with_tooltips(self):
-    from snippet.fields_with_tooltips import Coordinate
-    from camelot.view.proxy.collection_proxy import CollectionProxy
-    coordinate = Coordinate()
-    admin = Coordinate.Admin(None, Coordinate)
-    proxy = CollectionProxy(admin, lambda:[coordinate], admin.getFields )
-    form = admin.create_form_view('Coordinate', proxy, 0, None)
-    self.grab_widget(form)
+    def test_fields_with_actions(self):
+        from snippet.fields_with_actions import Coordinate
+        from camelot.view.proxy.collection_proxy import CollectionProxy
+        coordinate = Coordinate()
+        admin = Coordinate.Admin(None, Coordinate)
+        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
+        form = admin.create_form_view('Coordinate', proxy, 0, None)
+        self.grab_widget(form)
+      
+    def test_fields_with_tooltips(self):
+        from snippet.fields_with_tooltips import Coordinate
+        from camelot.view.proxy.collection_proxy import CollectionProxy
+        coordinate = Coordinate()
+        admin = Coordinate.Admin(None, Coordinate)
+        proxy = CollectionProxy(admin, lambda:[coordinate], admin.get_fields )
+        form = admin.create_form_view('Coordinate', proxy, 0, None)
+        self.grab_widget(form)
+        
+    def test_entity_validator(self):
+        from camelot.view.proxy.collection_proxy import CollectionProxy
+        from camelot.model.authentication import Person
+        from camelot.admin.application_admin import ApplicationAdmin
+        from snippet.entity_validator import PersonValidator, Admin
+        app_admin = ApplicationAdmin()
+        person_admin = Admin(app_admin, Person)
+        proxy = CollectionProxy(person_admin, lambda:[Person()], person_admin.get_columns)
+        validator = PersonValidator(person_admin, proxy)
+        self.mt.post(lambda:validator.isValid(0))
+        self.process()
+        self.assertEqual(len(validator.validityMessages(0)), 3)
+        self.grab_widget(validator.validityDialog(0, parent=None))
+        
+    def test_background_color(self):
+        from camelot.view.proxy.collection_proxy import CollectionProxy
+        from camelot.model.authentication import Person
+        from camelot.admin.application_admin import ApplicationAdmin
+        from snippet.background_color import Admin
+        app_admin = ApplicationAdmin()
+        person_admin = Admin(app_admin, Person)
+        proxy = CollectionProxy(person_admin, lambda:[Person(first_name='John', last_name='Cleese'),
+                                                      Person(first_name='eric', last_name='Idle')],
+                                                       person_admin.get_columns)
+        from camelot.view.controls.editors.one2manyeditor import One2ManyEditor
+        editor = One2ManyEditor(admin=person_admin)
+        editor.set_value(proxy)
+        self.process()
+        self.grab_widget(editor)
   
 class CamelotSchemaTest(SchemaTest):
   
-  images_path = static_images_path
+    images_path = static_images_path
   
 
 if __name__ == '__main__':
-  logger.info('running unit tests')
-  app = get_application()
-  runner=unittest.TextTestRunner(verbosity=2)
-  snippets_test =  unittest.makeSuite(SnippetsTest, 'test')
-  runner.run(snippets_test)
+    logger.info('running unit tests')
+    app = get_application()
+    runner=unittest.TextTestRunner(verbosity=2)
+    snippets_test =  unittest.makeSuite(SnippetsTest, 'test')
+    runner.run(snippets_test)
 #  proxy_test =  unittest.makeSuite(ProxyEntityTest, 'test')
 #  runner.run(proxy_test)
 #  proxy_one_to_many_test =  unittest.makeSuite(ProxyOneToManyTest, 'test')
 #  runner.run(proxy_one_to_many_test)
-  schema_test =  unittest.makeSuite(CamelotSchemaTest, 'test')
-  runner.run(schema_test)
-
-  form_action_test =  unittest.makeSuite(FormActionTest, 'test')
-  runner.run(form_action_test)
-  editors_test =  unittest.makeSuite(EditorsTest, 'test')
-  runner.run(editors_test)
-  editor_test =  unittest.makeSuite(DelegateTest, 'test')
-  runner=unittest.TextTestRunner(verbosity=2)
-  runner.run(editor_test)
-  controls_test = unittest.makeSuite(ControlsTest, 'test')
-  runner.run(controls_test)
-  form_test = unittest.makeSuite(FormTest, 'test')
-  runner.run(form_test)
-  filter_test = unittest.makeSuite(FilterTest, 'test')
-  runner.run(filter_test)    
-  entity_views_test = unittest.makeSuite(CamelotEntityViewsTest, 'test')
-  runner.run(entity_views_test)
+    schema_test =  unittest.makeSuite(CamelotSchemaTest, 'test')
+    runner.run(schema_test)
+    
+    form_action_test =  unittest.makeSuite(FormActionTest, 'test')
+    runner.run(form_action_test)
+    editors_test =  unittest.makeSuite(EditorsTest, 'test')
+    runner.run(editors_test)
+    editor_test =  unittest.makeSuite(DelegateTest, 'test')
+    runner=unittest.TextTestRunner(verbosity=2)
+    runner.run(editor_test)
+    controls_test = unittest.makeSuite(ControlsTest, 'test')
+    runner.run(controls_test)
+    form_test = unittest.makeSuite(FormTest, 'test')
+    runner.run(form_test)
+    filter_test = unittest.makeSuite(FilterTest, 'test')
+    runner.run(filter_test)    
+    entity_views_test = unittest.makeSuite(CamelotEntityViewsTest, 'test')
+    runner.run(entity_views_test)
 
